const tool = require('curl-ganteng');
const Cacheman = require('cacheman');
const GD = function () {

}

GD.prototype.setID = function (id) {
  this.id = id;
};
GD.prototype.setTTL = function(time)  {
  this.cache = new Cacheman('direct-gd', {ttl: time ? time : '3h', engine: 'file'});
}
GD.prototype.setLink = function (link) {
  this.id = link.match(/https?:\/\/(?:[\w\-]+\.)*(?:drive|docs)\.google\.com\/(?:(?:folderview|open|uc)\?(?:[\w\-\%]+=[\w\-\%]*&)*id=|(?:folder|file|document|presentation|spreadsheets)\/d\/|spreadsheet\/ccc\?(?:[\w\-\%]+=[\w\-\%]*&)*key=)([\w\-]{28,})/i)[1];
};

GD.prototype.get = function () {
  let cache = this.cache;
  if(!cache) cache = new Cacheman('direct-gd', {ttl: '3h', engine: 'file'});
  let current = this;
  if(!current.id) return new Promise((resolve, reject) => reject({message: `Invalid google drive ID`}));
  return cache.get(this.id).then((cached) => {
    if(cached) return cached;
    return tool.curl(`https://drive.google.com/uc?export=download&id=${current.id}`, {redirect: false}).then((res) => {
      if(res.headers_obj.Location) {
        cache.set(current.id, {link: res.headers_obj.Location}).then((done) => {
        }).catch((err) => {
          console.log('error saving cache', err);
        })
        return {link: res.headers_obj.Location}
      }
      if(/ServiceLogin/ig.test(res.req_url)) throw {message: 'forbidden', code: 403};
      let data = getData(res.body);
      return tool.curl(data.link, {cookie: res.cookie, redirect: false}).then((done) => {
        let output = done.headers_obj.Location;
        if(!output) throw {message: 'Direct link not found'};
        cache.set(current.id, {fileName: data.fileName, link: output}).then((done) => {

        }).catch((err) => {
          console.log('error saving cache', err);
        })
        return {fileName: data.fileName, link: output};
      })
    });
  })
};

function getData(body) {
  if(/Error 404/ig.test(body)) throw {message: 'file removed', code: 404};
  if(/Too many users have viewed or downloaded this file recently|Baru-baru ini terlalu banyak pengguna yang telah melihat atau mendownload file ini|Ce fichier a été récemment consulté ou téléchargé par un trop grand nombre/ig.test(body)) throw ({message: 'file limit', code: 100})
  let $ = tool.jquery(body);
  let link = $('#uc-download-link').attr('href');
  let fileName = $('.uc-name-size').children('a').text().trim();
  if(!link) throw new Error(`Can't get link from download button`);
  return {fileName, link: 'https://drive.google.com' + link};
}

module.exports = GD;
